.module/ram/abs=0      C18A;
.pagelength  59;                    { Works for .LST files with 550C }
.include    <system.k>;

{******************************************************************************
 *
 *      EZ-KIT Lite DSP Program for SOSSD
 *	c18a.dsp  -  DSP for 18 MHz 5 Watter.
 *	RSL 2 October 1999
 *
 *****************************************************************************}

{***********************   CONSTANTS     *************************************}
.const	    MAXBCOUNT = 100;

.const	    GAIN_I=32400;
.const      GAIN_Q=32767;
.const      GAIN_IQ=2000;

.const      GAIN_2X_I=0;
.const      GAIN_2X_Q=0;

.const	    RGAIN_I=32767;
.const      RGAIN_Q=24000;
.const      RGAIN_IQ=1500;

.const      CW_FREQ=1092;		{ 800 Hz }
.const      T_R_COUNT0=10000;
.const	    T_R_COUNT1=12000;		{ 20 uS * 10000 0.2 sec }
.const      TAPS1=130;                  { FIR1 - 2.8 KHz LPF, 48kHz rate}
.const      TAPS5=200;                  { FIR5 - .5KHz LPF, CW Xmit shaping }
.const	    DELAY_SIZE=512;
.const      H3=247;			{ Num taps in Hilbert #3 FIR filt }
.const      H3P1ON2=124;		{ This is  (H3+1)/2  }
.const      H3M1ON2=123;		{ ...and (H3-1)/2    }
.const      H3M3ON2=122;		{ ...and (H3-3)/2    }

{********************   PROGRAM MEMORY     ***********************************}

.var/pm/circ    fir1_coefs[TAPS1];      { 2.8 kHz LPF }
.init           fir1_coefs: <lp2_8.dat>;

.var/pm/circ    fir2_coefs[TAPS1];      { 500 Hz CW BPF }
.init           fir2_coefs: <bpcw1.dat>;

.var/pm/circ    fir5_coefs[TAPS5];      { CW transmit filter }
.init           fir5_coefs: <lp_5_48.dat>;

.var/pm/circ	hilbert3_coeff[H3P1ON2];
.init		hilbert3_coeff: <hil_3_48.dat>;

.var/pm         sin_coeff[5];
.init           sin_coeff: H#324000, H#005300, H#AACC00, H#08B700, H#1CCE00;

.var/pm		aud_gain[4];
.init		aud_gain: H#7FFF00, H#4C3F00, H#5A9E00, H#6BB300;
{               Audio gain:   0dB,   -4.5dB    -3.0dB,   -1.5dB }

{********************   DATA MEMORY     **************************************}
{ SPORT0, i.e., Serial PORT 0, is located in data memory.
  The AD1847 Codec hardware operates independently of the DSP.
  But when A/D conversion is complete there is an interrupt generated
  by SPORT0.  This occurs 48,000 times per second as determined
  by the crystal clock in the Codec.  This interrupt causes, among other
  things, a jump to location 14 Hex in program memory.  This, in turn,
  is sent off to process the newly received data, or to do any other task
  required on a periodic basis, such as generation of waveforms.
  It is at this time that new data can be sent to the Codec to be converted
  by the D/A in the Codec.  Memory locations tx_buf and rx_buf provide the
  interaction with the SPORT0 (and thus the Codec) when interrupts occur. }
.var/dm/circ	tx_buf[3];		{ SPORT0: Cmd + L data + R data }
.var/dm/circ 	rx_buf[3];		{ SPORT0: Status + L data + R data }

{ Initialization of the AD1847 Codec is complex.  The internal Codec
  registers are transfered out of the following buffer.  It is circular to
  allow detection of a complete load of commands.     }
.var/dm/circ    init_cmds[13];		{ Codec Initialization }
.var/dm		stat_flag;		{ Codec Init status }

.var/dm		xmit_power;		{ 0 to 32767, multiply for cw, ssb }
.var/dm		mike_gain;		{ 0 to 32767, g/4 before lpf }
.var/dm		af_gain_set;		{ 0 to 76, 1.5 dB per step }
.var/dm		af_shift;		{ -10 to +9, 6 dB shift steps }
.var/dm		af_gain;		{ Voltage gain, modified by af_shift }
{ t_r_control is used to keep the transmitter on an extra .1 sec after key
  comes up. It is a counter, 0 to 5000. }
.var/dm		t_r_control;		{ 0 to 4999=transmit; 5000=receive }
.var/dm		key;			{ 0=key down (on), 1=key up }

.var/dm		sdtone_ampl;
.var/dm		agc_decay;		{ Proportion of AGC error }
.var/dm		agc_gain;		{ 9.7 format 1 to 255 }
.var/dm		audio_env;
.var/dm/circ	delay[DELAY_SIZE];	{ The delay line, binaural }
.var/dm		del_i0_sav;		{ Storage when not in interrupt }

.var/dm/circ	idata[TAPS1];
.var/dm/circ	qdata[TAPS1];
.var/dm/circ	xmt_cw_data[TAPS5];
.var/dm/circ	h3delay[H3M1ON2];
.var/dm/circ	h3data[H3];
.var/dm/circ	xmtdata[TAPS1];
.var/dm		mode_select;		{ 0=ssb, 1=CW }
.var/dm		filter_select;		{ 0=3 kHz, 1=CW Narrow }
.var/dm         i0_sav, m0_sav, l0_sav; { Used by interrupting routine }
.var/dm		m1_sav;
.var/dm         i4_sav, m4_sav, l4_sav; {   to save fft dedicated DAG regs }
.var/dm         fir1i_i0_sav;		{ Save DAG0 reg FIR1 }
.var/dm         fir1q_i0_sav;
.var/dm		firxmt_i0_sav;
.var/dm         fir5_i0_sav;		{ Save DAG0 reg FIR5 }
.var/dm		save_i;			{ Temporary audio storage }
.var/dm         cw_dphase;
.var/dm		cw_phase;
.var/dm		h3delay_i0_sav;
.var/dm		h3data_i0_sav;

.var/dm		button1, button2, button3, button4;
.var/dm		bcount1, bcount2, bcount3, bcount4;

.var/dm		t1, t2, t3, t4, t5;		{ Temporary storage }

{********************  Interrupt vector table  *******************************}
	{ See c1shell.dsp for comments on the following }
        jump start;  rti; rti; rti;     {00: reset }
        rti;         rti; rti; rti;     {04: IRQ2 }	{ Filler instr }
        rti;         rti; rti; rti;     {08: IRQL1 }
        rti;         rti; rti; rti;     {0c: IRQL0 }
	ar = dm(stat_flag);		{10: SPORT0 tx }
        ar = pass ar;			{  see if needs initialization }
        if eq rti;			{ If dm(stat_flag)==0 exit this int }
        jump next_cmd;			{   else go transfer a control word }

	{ The following instructions are executed when an A/D conversion
	  cycle has been completed in the AD1847 Codec, and the resulting data
	  has been serially transmitted to the SPORT0.  The SPORT0 hardware
	  then generates an interrupt with an associated jump to this
	  program address: }
	jump input_samples;             {14: SPORT0 rx }
                     rti; rti; rti;	{Filler so that there are 4 instr }

	{ And more unused interrupt locations: }
        rti;         rti; rti; rti;     {18: IRQE }
        rti;         rti; rti; rti;     {1c: BDMA }
	rti;         rti; rti; rti;     {20: SPORT1 tx or IRQ1 }
        rti;         rti; rti; rti;     {24: SPORT1 rx or IRQ0 }
	rti;         rti; rti; rti;     {28: timer }
        rti;         rti; rti; rti;     {2c: power down }

{*****************************  START  ***************************************
 * Initialize process and drop into main loop.  None of the index registers
 * have been initialized, so we need to turn off all interrupts before
 * the hardware is allowed to generate one.  Then we are safe with instructions
 * like dm(i0, m0)=ar.  This is the dreaded uninitialized pointer problem!!
 ****************************************************************************}
start:	imask=0;			{ Turn off all interrupts }
	call init0;			{ Instructions that simulate easily }

	reset FL1;
	ax0 = CW_FREQ;			{ 800 Hz }
	dm(cw_dphase) = ax0;
	ax0 = 32760;
	dm(xmit_power) = ax0;
	dm(mike_gain) = ax0;

	ax0 = 0;
	dm(mode_select) = ax0;		{ SSB mode }

	ax0 = 5000;			{ Receive }
	dm(t_r_control) = ax0;

	ax0 = 1;			{ Key up (Not sending) }
	dm(key) = ax0;

	ax0 = 0;
	dm(filter_select) = ax0;	{ Wideband }

	ax0 = B#0111101101000000;	{ Set PF6 to output (see AD Data ) }
	dm(PFTYPE) = ax0;		{ Defined in system.k }

        i0=^idata;  m0=1; l0=TAPS1;	{ Output of I mixer }
        dm(fir1i_i0_sav)=i0;		{ Save i0 for interrupt }
        cntr=TAPS1;
        do zero1i until ce;
zero1i:      dm(i0,m0)=0;                { clear 'data' fir delay line buffer }
        i0=^qdata;  m0=1; l0=TAPS1;	{ Output of Q mixer }
        dm(fir1q_i0_sav)=i0;		{ Save i0 for interrupt; protects fft }
        cntr=TAPS1;
        do zero1q until ce;
zero1q:      dm(i0,m0)=0;		{ clear 'data' fir delay line buffer }

        i0=^xmtdata;  m0=1; l0=TAPS1;	{ Transmit audio }
        dm(firxmt_i0_sav)=i0;		{ Save i0 for interrupt }
        cntr=TAPS1;
        do zeroxmt until ce;
zeroxmt:    dm(i0,m0)=0;                { clear 'data' fir delay line buffer }

        i0=^xmt_cw_data; m0=1; l0=TAPS5; { CW Spectrum shaping }
        dm(fir5_i0_sav)=i0;
        cntr=TAPS5;
        do zero5 until ce;
zero5:       dm(i0,m0)=0;		{ clear fir5 delay line buffer }

	i0=^h3delay; m0=1; l0=%h3delay;
	dm(h3delay_i0_sav)=i0;
	cntr=%h3delay;
	do zeroh3dly until ce;
zeroh3dly:  dm(i0,m0)=0;

	i0=^h3data; m0=1; l0=%h3data;
	dm(h3data_i0_sav)=i0;
	cntr=%h3data;
	do zeroh3dat until ce;
zeroh3dat:  dm(i0,m0)=0;

	ax0 = ^delay;			{ Initialize delay line pointer }
	dm(del_i0_sav) = ax0;

	ax0 = 40;			{ Gain -15 dB }
	dm(af_gain_set) = ax0;
	call afgain;			{ Sets dm(af_shift) and dm(af_gain) }


	{ Be sure that all pointers that can be used in the interrupt
	  routines, like dm(fir1q_i0_sav) are initialized before
	  calling init1. It (init1) enables the interrupts and will
	  write randomly into memory if this is not done.
	}
	call init1;			{ And those that do not simulate }

{****************     Main Program Loop    ************************************
 *  Main processing loop.  Does all non-periodic (background) processing.
 *  This loop is interrupted at a 48KHz rate to send and receive Codec data.
 *****************************************************************************}
again:
	{*******************  Button 1 - AF Gain Up   *************************}
	ar = dm(bcount1);		{ Update pushbutton 1 status }
	none = pass ar;			{ See if count is 0 }
	if eq jump pb1w;		{ Yes, it is 0 }
	ay0 = MAXBCOUNT;		{ See if at max count }
	none = ar - ay0;
	if eq jump pb1x;		{ At max count }
	jump pb1z;			{ Do nothing, count is in between }
	{ If the count=0 and the state=0, do nothing. If state=1, reset to 0 }
pb1w:	ar = dm(button1);		{ Here because count==0 }
	none = pass ar;			{ Get button state }
	if eq jump pb1z;		{ State still 0, so do nothing }
	ar = 0;
	dm(button1) = ar;		{ Set the state to 0 }

	{ Here is the place to put any operation that should occur
	  each time the button is pushed }
	ar = dm(af_gain_set);
	ay0 = 76;			{ Highest gain value }
	none = ar - ay0;		{ See if we are there }
	if ge jump pb1z;		{ Yes, so do nothing }
	ar = ar + 1;
	dm(af_gain_set) = ar;
	call afgain;
	jump pb1z;
	{ If the count=max, and state=1, do nothing. If state=0, set to 1 }
pb1x:	ar = dm(button1);		{ We are at max count }
	none = pass ar;			{ Test state }
	if ne jump pb1z;
	ar = 1;
	dm(button1) = ar;		{ Set the state to 1 }
pb1z: 					{  All done }
	{*****************   Button 2 - AF Gain Down   **********************}
	ar = dm(bcount2);		{ Update pushbutton 2 status }
	none = pass ar;			{ See if count is 0 }
	if eq jump pb2w;		{ Yes, it is 0 }
	ay0 = MAXBCOUNT;		{ See if at max count }
	none = ar - ay0;
	if eq jump pb2x;		{ At max count }
	jump pb2z;			{ Do nothing, count is in between }
	{ If the count=0 and the state=0, do nothing. If state=1, reset to 0 }
pb2w:	ar = dm(button2);		{ Here because count==0 }
	none = pass ar;			{ Get button state }
	if eq jump pb2z;		{ State still 0, so do nothing }
	ar = 0;
	dm(button2) = ar;		{ Set the state to 0 }

	ar = dm(af_gain_set);
	none = pass ar;			{ See if gain is at 0, already }
	if le jump pb2z;		{ Yes, so do nothing }
	ar = ar - 1;
	dm(af_gain_set) = ar;
	call afgain;
	jump pb2z;

	{ If the count=max, and state=1, do nothing. If state=0, set to 1 }
pb2x:	ar = dm(button2);		{ We are at max count }
	none = pass ar;			{ Test state }
	if ne jump pb2z;
	ar = 1;
	dm(button2) = ar;		{ Set the state to 1 }
pb2z: 					{  All done }

	{*****************  Button 3 - Filter Select  **********************}
	ar = dm(bcount3);		{ Update pushbutton 3 status }
	none = pass ar;			{ See if count is 0 }
	if eq jump pb3w;		{ Yes, it is 0 }
	ay0 = MAXBCOUNT;		{ See if at max count }
	none = ar - ay0;
	if eq jump pb3x;		{ At max count }
	jump pb3z;			{ Do nothing, count is in between }
	{ If the count=0 and the state=0, do nothing. If state=1, reset to 0 }
pb3w:	ar = dm(button3);		{ Here because count==0 }
	none = pass ar;			{ Get button state }
	if eq jump pb3z;		{ State still 0, so do nothing }
	ar = 0;
	dm(button3) = ar;		{ Set the state to 0 }

	ar = dm(filter_select);
	ay0 = 1;
	ar = ar xor ay0;
	dm(filter_select) = ar;
	jump pb3z;

	{ If the count=max, and state=1, do nothing. If state=0, set to 1 }
pb3x:	ar = dm(button3);		{ We are at max count }
	none = pass ar;			{ Test state }
	if ne jump pb3z;
	ar = 1;
	dm(button3) = ar;		{ Set the state to 1 }
pb3z: 					{  All done }

	{*************   Button 4 - Transmit Mode CW/SSB   *****************}
	ar = dm(bcount4);		{ Update pushbutton 4 status }
	none = pass ar;			{ See if count is 0 }
	if eq jump pb4w;		{ Yes, it is 0 }
	ay0 = MAXBCOUNT;		{ See if at max count }
	none = ar - ay0;
	if eq jump pb4x;		{ At max count }
	jump pb4z;			{ Do nothing, count is in between }
	{ If the count=0 and the state=0, do nothing. If state=1, reset to 0 }
pb4w:	ar = dm(button4);		{ Here because count==0 }
	none = pass ar;			{ Get button state }
	if eq jump pb4z;		{ State still 0, so do nothing }
	ar = 0;
	dm(button4) = ar;		{ Set the state to 0 }

	ar = dm(mode_select);
	ay0 = 1;
	ar = ar xor ay0;
	dm(mode_select) = ar;
	jump pb4z;

	{ If the count=max, and state=1, do nothing. If state=0, set to 1 }
pb4x:	ar = dm(button4);		{ We are at max count }
	none = pass ar;			{ Test state }
	if ne jump pb4z;
	ar = 1;
	dm(button4) = ar;		{ Set the state to 1 }
pb4z: 					{  All done }

	{ Transmit/Receive Control - Controled by dm(t_r_control) according to:
	     0 to T_R_COUNT0-1=Transmit
             T_R_COUNT0 to T_R_COUNT1-1=Receive without headphones
             T_R_COUNT1=Receive with headphones connected
        }
	ar = dm(t_r_control);		{ Check the T/R status }
	ay0 = T_R_COUNT0-1;
	none = ar - ay0;		{ Check the value of t_r_control }
	if le jump trc1;		{ In transmit, or holding delay }
	reset FL0;			{ Voltage to +10R, not +10T }
	ay0 = T_R_COUNT1-1;
	none = ar - ay0;		{ Check to see if receive w/o audio }
	if le jump trc2;

	{ Here because dm(t_r_control)==T_R_COUNT1, i.e., full receive }
	ar = T_R_COUNT1;		{ Make sure count can't wrap }
	dm(t_r_control) = ar;
	ax0 = B#00000000;		{ Make sure headphones connected }
	dm(PFDATA) = ax0;
	jump trc3;

	{ Here because dm(t_r_control)==0, i.e., full transmit }
trc1:	set FL0;			{ Voltage to +10T, not +10R }
	ax0 = B#01000000;		{ Make sure headphones disconnected }
	dm(PFDATA) = ax0;
	jump trc3;

	{ Hr because T_R_COUNT0<=dm(t_r_count)<T_R_COUNT1, receive audio dly}
trc2:	ax0 = B#01000000;		{ Set PF6 to 1 to keep audio off }
	dm(PFDATA) = ax0;		{ Set and fall through to trc3 }

trc3:	jump again;			{ Go round and round forever }

{***********************************************************************
 *                 SPORT0 Interrupt Handler                            *
 *  SPORT0 inputs from the Codec put us here 48000 times per second.   *
 *  Routines here are those that need this periodic service.           *
 ***********************************************************************}
input_samples:
        ena sec_reg;                    { use secondary register bank }
        dm(i0_sav)=i0; dm(m0_sav)=m0; dm(l0_sav)=l0; { Free up DAG 0 & 4 }
        dm(i4_sav)=i4; dm(m4_sav)=m4; dm(l4_sav)=l4;

	{Now look send/receive line and act accordingly. Note this is done
	 here so that we do not need to deal with having an interrupt when
	 we are half-way through this process.
	 Go to transmit immediately with t_r_control=0. Decay up to receive
         by adding one to it each time through here.
	}
	ax0 = dm(PFDATA);
	af = tstbit 4 of ax0;
	if ne jump tr0;
	ar = 0;				{ Set to Transmit }
	dm(t_r_control) = ar;		{ To be checked in background }
	set FL0;
	jump tr1;
tr0:	ay0 = dm(t_r_control);		{ Here because we want receive }
	ar = ay0 + 1;			{ Continue time out towards receive }
	dm(t_r_control) = ar;		{ To be checked in the background }
tr1:
	{ Now essentially the same thing for the CW key line }
	ax0 = dm(PFDATA);
	af = tstbit 5 of ax0;
	if ne jump key0;
	ar = 0;				{ Key down; send CW }
	dm(key) = ar;
	dm(t_r_control) = ar;		{ And put rig into transmit }
	jump key1;
key0:	ar = 1;
	dm(key) = ar;
key1:
	{ Audio Processing: }
	mx0 = dm(rx_buf+1);		{ Get af sample from SPORT0 (codec) }
	mx1 = dm(rx_buf+2);		{   and get quadrature sample }
	ax0 = dm(t_r_control);		{ Bit 0 = 1 is xmt }
	none = pass ax0;
	if eq jump xmt_isr;		{ Do xmit functions }

	{ To finish the SSB reception process, we need to filter the I and Q
	  audio signals, put a 90 degree relative phase shift on them, and
	  add or subtract to get the desired sideband. }

	i0=dm(fir1i_i0_sav);  m0=1;  l0=%idata;
	dm(i0,m0) = mx0;		{ Store sample in fir1 delay line }
        dm(fir1i_i0_sav)=i0;  		{ i0 for next call of do_fir1i }

	i0=dm(fir1q_i0_sav);  m0=1;  l0=%qdata;
	dm(i0,m0) = mx1;		{ Store sample in fir1 delay line }
        dm(fir1q_i0_sav)=i0;  		{ i0 for next call of do_fir1q }

	ar = dm(filter_select);		{ Button for narrow or wide filter }
	none = pass ar;
	if gt jump filsl1;
	call do_fir1i;			{ LPF for I, out in dm(save_i) }
	call do_fir1q;			{ LPF for Q, out in sr0 }
	jump filsl2;
filsl1:	call do_fir2i;			{ CW BPF for I, out in dm(save_i) }
	call do_fir2q;			{ CW BPF for Q, out in sr0 }
filsl2:

	ar = dm(save_i);
	my1 = RGAIN_I;			{ Gain correction factor }
	mr = ar * my1 (SS);		{ I signal * correction }
	dm(save_i) = mr1;
	my1 = RGAIN_IQ;
	mr = ar * my1 (SS);
	ay0 = mr1;			{ Cross-correction for Q }
	my1 = RGAIN_Q;			{ Q chan gain correction }
	mr = sr0 * my1 (SS);		{ Q signal * correction }
	ar = mr1 + ay0;			{ Add in cross-correction }

	mr1=dm(save_i);			{ For hilbert }
	call do_hilbert;		{ 90 deg; ar, mr1 in; ax1, mr1 out }
	ay1 = mr1;			{ Q out }
	ar = ax1 - ay1;			{ - = usb }

	my0 = dm(af_gain);
	mr = ar * my0 (SS);
	se = dm(af_shift);		{ Number of shifts }
	sr = ashift mr1 (hi);
	dm(tx_buf+1) = sr1;		{ Audio output }

	{ Delay of right channel by amount delay_right.  If set to 1 there is
	  no delay. }
	i0=dm(del_i0_sav); l0=DELAY_SIZE;
	m0=0;
	mr1=dm(i0, m0);			{ Remove the delayed signal }
	m0=1;
	dm(i0, m0)=sr1;			{ Put the new signal into the dly line}
	dm(del_i0_sav)=i0;		{ Save the pointer for next time }
	dm(tx_buf+2) = mr1;		{ Right }

end_blocks:
	{*****************  Button 1  -  Up  *******************}
	ax0 = dm(PFDATA);
	ar = tstbit 0 of ax0;		{ See if the button is pushed }
	if ne jump bu1a;		{ Button input high (not pushed) }
	{ Here because the button was pushed, causing the input to go low }
	ay0 = dm(bcount1);		{ Get ready for count down }
	ar = ay0 -1;			{ Button input is low (pushed) }
	if lt jump bu1c;		{ Don't let it go neg, no change }
	dm(bcount1) = ar;
	jump bu1b;			{ Go save reduced count }
	{ Here because the button was not pushed; the input is high }
bu1a:	ay0 = dm(bcount1);		{ Get ready for count up }
	ar = ay0 + 1;			{ Button input is high }
	ay0 = MAXBCOUNT;
	none = ar - ay0;		{ Don't allow over MAXBCOUNT }
	if gt jump bu1c;		{ No change needed to count }
bu1b:	dm(bcount1) = ar;		{ Here to update count }
bu1c:
	{*****************  Button 2  -  Down  *******************}
	ax0 = dm(PFDATA);
	ar = tstbit 1 of ax0;		{ See if the button is pushed }
	if ne jump bu2a;		{ Button input high (not pushed) }
	{ Here because the button was pushed, causing the input to go low }
	ay0 = dm(bcount2);		{ Get ready for count down }
	ar = ay0 -1;			{ Button input is low (pushed) }
	if lt jump bu2c;		{ Don't let it go neg, no change }
	dm(bcount2) = ar;
	jump bu2b;			{ Go save reduced count }
	{ Here because the button was not pushed; the input is high }
bu2a:	ay0 = dm(bcount2);		{ Get ready for count up }
	ar = ay0 + 1;			{ Button input is high }
	ay0 = MAXBCOUNT;
	none = ar - ay0;		{ Don't allow over MAXBCOUNT }
	if gt jump bu2c;		{ No change needed to count }
bu2b:	dm(bcount2) = ar;		{ Update count }
bu2c:
	{*****************  Button 3  -  Filter Select  *******************}
	ax0 = dm(PFDATA);
	ar = tstbit 2 of ax0;		{ See if the button is pushed }
	if ne jump bu3a;		{ Button input high (not pushed) }
	{ Here because the button was pushed, causing the input to go low }
	ay0 = dm(bcount3);		{ Get ready for count down }
	ar = ay0 -1;			{ Button input is low (pushed) }
	if lt jump bu3c;		{ Don't let it go neg, no change }
	dm(bcount3) = ar;
	jump bu3b;			{ Go save reduced count }
	{ Here because the button was not pushed; the input is high }
bu3a:	ay0 = dm(bcount3);		{ Get ready for count up }
	ar = ay0 + 1;			{ Button input is high }
	ay0 = MAXBCOUNT;
	none = ar - ay0;		{ Don't allow over MAXBCOUNT }
	if gt jump bu3c;		{ No change needed to count }
bu3b:	dm(bcount3) = ar;		{ Update count }
bu3c:
	{*****************  Button 4  -  Mode USB/CW  *******************}
	ax0 = dm(PFDATA);
	ar = tstbit 3 of ax0;		{ See if the button is pushed }
	if ne jump bu4a;		{ Button input high (not pushed) }
	{ Here because the button was pushed, causing the input to go low }
	ay0 = dm(bcount4);		{ Get ready for count down }
	ar = ay0 -1;			{ Button input is low (pushed) }
	if lt jump bu4c;		{ Don't let it go neg, no change }
	dm(bcount4) = ar;
	jump bu4b;			{ Go save reduced count }
	{ Here because the button was not pushed; the input is high }
bu4a:	ay0 = dm(bcount4);		{ Get ready for count up }
	ar = ay0 + 1;			{ Button input is high }
	ay0 = MAXBCOUNT;
	none = ar - ay0;		{ Don't allow over MAXBCOUNT }
	if gt jump bu4c;		{ No change needed to count }
bu4b:	dm(bcount4) = ar;		{ Update count }
bu4c:

	i0=dm(i0_sav); m0=dm(m0_sav); l0=dm(l0_sav); { Restore DAG 0 & 4 }
        i4=dm(i4_sav); m4=dm(m4_sav); l4=dm(l4_sav);
        dis sec_reg;
	rti;				{ This undoes the interrupt }
{ ************************************************************************ }

{ xmt_isr  -  Routine jumped to when in transmit. Comes with audio signal 
  in mx0.  Handles CW, USB.
}
xmt_isr:
	ar=dm(mode_select);
	none = pass ar;
	if eq jump xi_ssb;

SET FL1;
xi_cw:
	{ If key is down, put a .9 (29491) into CW fir filt.
	  Modulate fir output onto carrier.  This scheme
	  allows top space for overshoot in the fir. }
	ax0 = dm(key);			{ Get hardware CW key data }
	none = pass ax0;
	ar = 0;				{ CW off }
	if ne jump xi_cw1;		{ CW key is up }
xi_cw0:	ar = 29491;			{ 0.9 to key click filter }
xi_cw1:	call fir_xmt_cw;		{ Input in ar, output in mr1 }

	my0 = dm(xmit_power);		{ Xmit_gain (power control) }
	mr = mr1 * my0 (SS);		{ Power controlled modulating signal }
	my0 = mr1;
	ax0 = dm(cw_dphase);		{ Phase increment for lo }
	ay0 = dm(cw_phase);		{ Last phase }
	ar = ax0 + ay0;			{ New phase }
	dm(cw_phase) = ar;		{ For next time }

ax0 = dm(cw_phase);
{ ax0 = ar; }
	call sin;		{ Phase in ax0, Sin returned in AR }
	mr=ar*my0(SS);			{ CW Gate }
	ar = -mr1;			{ Make USB}

	my1 = GAIN_I;			{ Gain correction factor }
	mr = ar * my1 (SS);		{ Keyed sine wave * correction }
	ar = mr1;			{ Corrected I signal }

	my1 = GAIN_IQ;
	mr = ar * my1 (SS);
	dm(t1) = mr1;			{ Cross-correction for Q }
	dm(tx_buf + 1) = ar;		{ In-phase transmit i-f sig out }

	{ That takes care of I, now Q: }
	ax0 = dm(cw_phase);		{ This is the phase used for I chan }
	ay0 = 16384;			{ 90 degrees for quadrature lo }
	ar = ax0 + ay0;
	ax0 = ar;  call sin;		{ Cos lo signal, sin() preserves my0 }
	mr = ar * my0(SS);		{ CW Gate for Q signal }
	my1 = GAIN_Q;			{ Q chan gain correction }
	mr = mr1 * my1 (SS);
	ay0 = dm(t1);			{ Now add in cross-correction }
	ar = mr1 + ay0;

	dm(tx_buf + 2) = ar;		{ Quadrature transmit sig out }
RESET FL1;
	jump end_blocks;

{ xi_ssb  -  All of the routines associated with ssb transmit.
  These are 48 kHz rate.
  Transmit audio comes from the the left adc (mx0) and is run through 
  hilbert3 to create I and Q components.
}
xi_ssb:
	{ Audio gain of 1 to here. To allow some gain in this path, put 
	  mike gain here (0, 32767/32768) and follow with x8.  Dynamic
	  range is not a problem here.
	  Note we are clippimg, filtering and then generating ssb. This is
	  not the best and one should avoid the clipping region when
	  using ssb.  Audio comes in mx0.        }
	my0 = dm(mike_gain);
	mr = mx0 * my0 (SS);		{ adc output reduced by mike gain }

	mx0 = mr1;
	my0 = 32767;			{ about 1.0 }
	mr = mr+mx0*my0 (SS);		{ x2 }
	mr = mr+mx0*my0 (SS);		{ x3 }
	mr = mr+mx0*my0 (SS);		{ x4 }
	mr = mr+mx0*my0 (SS);		{ x5 }
	mr = mr+mx0*my0 (SS);		{ x6 }
	mr = mr+mx0*my0 (SS);		{ x7 }
	mr = mr+mx0*my0 (SS);		{ x8 }
	{ Do it this way to clip w/o wrapping around to opposite polarity }
	if mv sat mr;			{ Clip to max +/- value }
	my0=dm(xmit_power);
	mr=mr1*my0 (SS);
	ar = mr1;

	call fir_xmt;			{ Input in ar, output in mr1 }

	{ Hilbert transform needs parallel inputs in ar and mr1 }
	ar = mr1;
	call do_hilbert;		{ 90 deg, returns I=ax1, Q=mr1 }
	ar = -ax1;			{ This gives USB }
	dm(tx_buf + 1) = ar;		{ I out }
	dm(tx_buf + 2) = mr1;		{ Q out }
	jump end_blocks;


{*************************************************************************
 *       Sine Approximation:     Y = Sin(x)                              *
 *       Calling Parameters:     AX0 = x in scaled 1.15 format           *
 *       Return Values:          AR = y in 1.15 format                   *
 *       Altered Registers:      AY0,AF,AR,MY1,MX1,MF,MR,SR,I4,M4,L4     *
 *       Computation Time:       25 cycles                               *
 * Thanks to Analog Devices Applications #1
 *************************************************************************}

sin:    M4=1;    L4=0;
        I4=^sin_coeff;                  { Pointer to coeff. buffer }
        AY0=H#4000;
        AR=AX0, AF=AX0 AND AY0;         { Check 2nd or 4th quad. }
        IF NE AR=-AX0;                  { If yes, negate input }
        AY0=H#7FFF;
        AR=AR AND AY0;                  { Remove sign bit }
        MY1=AR;
        MF=AR*MY1 (RND), MX1=PM(I4,M4); { MF = x**2 }
        MR=MX1*MY1 (SS), MX1=PM(I4,M4); { MR = C1*x }
        CNTR=3;
        DO approx UNTIL CE;
                MR=MR+MX1*MF (SS);
approx:         MF=AR*MF (RND), MX1=PM(I4,M4);
        MR=MR+MX1*MF (SS);
        SR=ASHIFT MR1 BY 3 (HI);
        SR=SR OR LSHIFT MR0 BY 3 (LO);  { Convert to 1.15 format }
        AR=PASS SR1;
        IF LT AR=PASS AY0;              { Saturate if needed }
        AF=PASS AX0;
        IF LT AR=-AR;                   { Negate output if needed }
        RTS;

{ Hilbert has independent inputs and outputs for delayed and
  phase shifted paths. Uses HIL_3_48.DAT running at 48 KHz
  in order to get response down to 300 Hz.
  Delayed path: ar in, ax1 out;  90 deg path: mr1 in, mr1 out.    }
do_hilbert:				{ 48 KHz Hilbert for receiving }
	dm(m1_sav) = m1;
	m1 = 1;

	i0 = dm(h3delay_i0_sav);  m0=0;  l0=%h3delay;
	ax1 = dm(i0, m0);		{ get ax1, the delayed output }
	dm(i0, m1) = ar;		{ Put new data in, update pointer }
	dm(h3delay_i0_sav) = i0;	{ Save pointer }

	i0=dm(h3data_i0_sav); m0=2; l0=%h3data;  { i0 points to data }
	i4=^hilbert3_coeff; m4=1; l4=%hilbert3_coeff;
	dm(i0, m1)=mr1;			{ Enter new data and bump pointer 1 }
	mr=0, mx0=dm(i0, m0), my0=pm(i4, m4);
	cntr=H3M3ON2;
	do hil3_loop until ce;
hil3_loop:   mr=mr+mx0*my0(SS), mx0=dm(i0, m0), my0=pm(i4, m4);
	mr=mr+mx0*my0(SS), mx0=dm(i0, m1), my0=pm(i4, m4);
	mr=mr+mx0*my0(RND);			{ mr1 = phase shifted output }
if mv sat mr;
	dm(h3data_i0_sav)=i0;

	m1 = dm(m1_sav);
	rts;

{*****************************************************************************
 * FIR1   A 2.8 kHz LPF with 80 dB down above 4KHz.  48 kHz sample rate
 *        Note - LP2_8.DAT has a gain of 4 that is removed after summing.
 * Inputs: Assumes input array has been updated and pointer is
 *            dm(fir1i_i0_sav) or q.
 * Output: Filtered output points in dm(save_i), and SR0.
 * Cycles: 13 + TAPS1  (143 cycles for TAPS1=130)
 *****************************************************************************}
do_fir1i:
	i0=dm(fir1i_i0_sav);  m0=1;  l0=%idata;
        i4=^fir1_coefs;      m4=1;  l4=%fir1_coefs;
        cntr=TAPS1-1;                   { 2.8 kHz LPF }
        mr=0, mx0=dm(i0,m0), my0=pm(i4,m4);
        do fir1iloop until ce;
fir1iloop:   mr=mr+mx0*my0(ss), mx0=dm(i0,m0), my0=pm(i4,m4);
        mr=mr+mx0*my0(rnd);		{ Filtered data in MR1 }
	if mv sat mr;
	{ Gain of fir, needs to be removed now }
	sr = ashift mr2 by -2 (hi);
	sr = sr or lshift mr1 by -2 (lo);
        dm(fir1i_i0_sav)=i0;		{ i0 for next call of do_fir1 }
	dm(save_i) = sr0;
	rts;

do_fir1q:
	i0=dm(fir1q_i0_sav);  m0=1;  l0=%qdata;
        i4=^fir1_coefs;      m4=1;  l4=%fir1_coefs;
        cntr=TAPS1-1;                   { 2.8 kHz LPF }
        mr=0, mx0=dm(i0,m0), my0=pm(i4,m4);
        do fir1qloop until ce;
fir1qloop:   mr=mr+mx0*my0(ss), mx0=dm(i0,m0), my0=pm(i4,m4);
        mr=mr+mx0*my0(rnd);		{ Filtered data in MR1 }
	if mv sat mr;
	{ Gain of fir needs to be removed now }
	sr = ashift mr2 by -2 (hi);
	sr = sr or lshift mr1 by -2 (lo);
        dm(fir1q_i0_sav)=i0;		{ i0 for next call of do_fir1 }
	rts;


{*****************************************************************************
 * FIR2   A 500 Hz BPF with 45 dB down above 4KHz.  48 kHz sample rate
 *        Note - BPCW1.DAT has a gain of 32 that is removed after summing.
 * Inputs: Assumes input array has been updated and pointer is
 *            dm(fir1i_i0_sav) or q. The data array is shared with fir1.
 * Output: Filtered output points in dm(save_i), and SR0.
 * Cycles: 13 + TAPS1  (143 cycles for TAPS1=130)
 *****************************************************************************}
do_fir2i:
	i0=dm(fir1i_i0_sav);  m0=1;  l0=%idata;
        i4=^fir2_coefs;      m4=1;  l4=%fir2_coefs;
        cntr=TAPS1-1;
        mr=0, mx0=dm(i0,m0), my0=pm(i4,m4);
        do fir2iloop until ce;
fir2iloop:   mr=mr+mx0*my0(ss), mx0=dm(i0,m0), my0=pm(i4,m4);
        mr=mr+mx0*my0(rnd);		{ Filtered data in MR1 }
	if mv sat mr;
	{ Gain of fir, needs to be removed now }
	sr = ashift mr2 by -5 (hi);
	sr = sr or lshift mr1 by -5 (lo);
        dm(fir1i_i0_sav)=i0;		{ i0 for next call of do_fir2i }
	dm(save_i) = sr0;
	rts;

do_fir2q:
	i0=dm(fir1q_i0_sav);  m0=1;  l0=%qdata;
        i4=^fir2_coefs;      m4=1;  l4=%fir1_coefs;
        cntr=TAPS1-1;
        mr=0, mx0=dm(i0,m0), my0=pm(i4,m4);
        do fir2qloop until ce;
fir2qloop:   mr=mr+mx0*my0(ss), mx0=dm(i0,m0), my0=pm(i4,m4);
        mr=mr+mx0*my0(rnd);		{ Filtered data in MR1 }
	if mv sat mr;
	{ Gain of fir needs to be removed now }
	sr = ashift mr2 by -5 (hi);
	sr = sr or lshift mr1 by -5 (lo);
        dm(fir1q_i0_sav)=i0;		{ i0 for next call of do_fir2q }
	rts;

{*****************************************************************************
 * FIR_XMT   A 2.8 kHz LPF with -80dB  above 4KHz.  48 kHz sample rate
 * Used to shape input audio for ssb transmission.
 * Note: This filter has a gain of 4 due to coefficient scaling.
 * Inputs: New data point in AR, pointer is dm(firxmt_i0_sav).
 * Output: Filtered output point in MR1.
 * Cycles: 13 + TAPS4  (143 cycles for TAPS4=130)
 *****************************************************************************}
fir_xmt:
sr=ashift ar by -2 (hi);
ar=sr1;					{UNTIL RESCALE COEFFICIENTS }
	i0=dm(firxmt_i0_sav);  m0=1;  l0=%xmtdata;
	dm(i0,m0) = ar;			{ Update delay line }
        i4=^fir1_coefs;        m4=1;  l4=%fir1_coefs;
        cntr=TAPS1-1;                   { 2.8 kHz LPF }
        mr=0, mx0=dm(i0,m0), my0=pm(i4,m4);
        do firxloop until ce;
firxloop:   mr=mr+mx0*my0(ss), mx0=dm(i0,m0), my0=pm(i4,m4);
        mr=mr+mx0*my0(rnd);		{ Filtered data in MR1 }
	if mv sat mr;

{	sr = ashift mr2 by -2 (hi);
	sr = sr or lshift mr1 by -2 (lo);
	mr1 = sr0;
}
        dm(firxmt_i0_sav)=i0;		{ i0 for next call of fir_xmt }
	rts;


{*****************************************************************************
 * FIR_XMT_CW LPF.  48 kHz sample rate
 * Used to shape keying signal for cw transmission.
 * Inputs: Data in AR and pointer is dm(fir5_i0_sav).
 * Output: Filtered output point in MR1.
 * Cycles: 13 + TAPS5  (213 cycles for TAPS5=200)
 *****************************************************************************}
fir_xmt_cw:
	i0=dm(fir5_i0_sav);  m0=1;  l0=%xmt_cw_data;
	dm(i0,m0) = ar;			{ Update delay line }
        i4=^fir5_coefs;      m4=1;  l4=%fir5_coefs;
        cntr=TAPS5-1;
        mr=0, mx0=dm(i0,m0), my0=pm(i4,m4);
        do fir5loop until ce;
fir5loop:   mr=mr+mx0*my0(ss), mx0=dm(i0,m0), my0=pm(i4,m4);
        mr=mr+mx0*my0(rnd);		{ Filtered data in MR1 }
	{ Gain of fir needs to be removed now }
	sr = ashift mr2 by -5 (hi);
	sr = sr or lshift mr1 by -5 (lo);
	mr1 = sr0;
        dm(fir5_i0_sav)=i0;		{ i0 for next call of fir_xmt_cw }
	rts;

{******************************************************************************
 * afgain()  -  Input is af_gain_set, an integer that ranges from 0 to 76.
 * Values above 40 are a net gain. Output is a shift value af_shift, that
 * ranges from -10 to +9 and is used to shift the audio signal right
 * or left in it's binary position, and af_gain a multiplier that provides
 * 4 1.5 dB steps between the af_shift values.
 *****************************************************************************}
afgain: ax0 = dm(af_gain_set);		{ The desired af gain setting }
	ay0 = H#0003;			{ Mask for LS 2 bits }
	ar = ax0 and ay0;		{ Mask the bits }
	ay1 = ^aud_gain;		{ The table of 1.5 dB steps }
	ar = ar + ay1;			{ Index to the table entry }
	i4 = ar;			{ To the index register }
	l4 = 0;				{ A non-circular table }
	m4 = 0;				{ No increment needed }
	ar = pm(i4, m4);		{ Get af_gain value }
	dm(af_gain) = ar;		{   and put it into af_gain }
	ay0 = 3;			{ Now find shift value }
	ar = ax0 + ay0;			{ Add 3 to af_gain_set }
	sr = ashift ar by -2 (hi);	{ This is divide by 4 }
	ay0 = 10;
	ar = sr1 - ay0;			{ Subtract 10 to get number of shifts }
	dm(af_shift) = ar;		{   and put it where we can use it }
	rts;				{ End of this routine }

{******************************************************************************
 * Main startup initialization.  Many variables and registers require
 * startup initialization, either for operation or to improve simulation.
 * This routine includes the parts that can easily be simulated
 *****************************************************************************}
init0:	{ i7, l7, i6, l6 devoted to auto-buffering in SPORT0.  Initialized
	  here. These need protection, if used elsewhere in the program.    }
        i7 = ^rx_buf;
        l7 = %rx_buf;
        m7 = 1;
        i6 = ^tx_buf;
        l6 = %tx_buf;
        m6 = 1;
	{ i5, l5, m5 used in next_cmd to load AD1847 regs and then made free. }
        i5 = ^init_cmds;
        l5 = %init_cmds;
        m5 = 1;

        ax0=0xC000; dm(tx_buf)=ax0;    { Mode Change Enable (MCE)  initially }
        ax0=0x0000; dm(tx_buf+1)=ax0;
        ax0=0x0000; dm(tx_buf+2)=ax0;

	{  AD1847  Command Register Initialization.  They are placed into
	   memory here and will be loaded into the AD1847 with interrupt
	   routines in init1.  The details of these registers are given
	   in the AD1847 data sheet.  Note that the destinations are given
	   by the second hex digit and that register addresses b, e and f
	   are not valid, or used.    }
        ax0=0xc002;  dm(init_cmds)=ax0;	   { Left Input control }
        ax0=0xc102;  dm(init_cmds+1)=ax0;  { Right Input control }
        ax0=0xc288;  dm(init_cmds+2)=ax0;  { Left Aux #1 control }
        ax0=0xc388;  dm(init_cmds+3)=ax0;  { Right Aux #1 control }
        ax0=0xc488;  dm(init_cmds+4)=ax0;  { Left Aux #2 control }
        ax0=0xc588;  dm(init_cmds+5)=ax0;  { Right Aux #2 control }
        ax0=0xc680;  dm(init_cmds+6)=ax0;  { Left DAC control }
        ax0=0xc780;  dm(init_cmds+7)=ax0;  { Right DAC control }
        ax0=0xc85c;  dm(init_cmds+8)=ax0;  { Format: 48 KHz, (c850h for 8 KHz)}
        ax0=0xc909;  dm(init_cmds+9)=ax0;  { Interface configuration }
        ax0=0xca00;  dm(init_cmds+10)=ax0; { Pin configuration }
        ax0=0xcc40;  dm(init_cmds+11)=ax0; { Misc Information }
        ax0=0xcd00;  dm(init_cmds+12)=ax0; { Digital Mix control }

{********************* Serial PORT 0 Initialization *************************
 * The following sets up the SPORT0 for autobuffering of 3 receive words
 * (Status, L data, and R data) or 3 transmit words (Command, L data, and
 * R data.)  These are data memory locations, like dm(SPORT0_Autobuf).
 * The actual locations in memory come from the system.k file and the details
 * of the data loaded to these dm locations is given in the Analog Devices
 * "ADSP-2100 Family Users Manual."
 *****************************************************************************}
        ax0= b#0000110101111111;   dm (SPORT0_Autobuf) = ax0;
        ax0 = 0;    dm (SPORT0_RFSDIV) = ax0;
        ax0 = 0;    dm (SPORT0_SCLKDIV) = ax0;
        ax0 = b#1000011000001111;   dm (SPORT0_Control_Reg) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_TX_Channels0) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_TX_Channels1) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_RX_Channels0) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_RX_Channels1) = ax0;

{******************* System and memory initialization ***********************
 * A variety of hardware dependent parameters are are set by the following
 * registers.  The first two are in data memory and ifc, icntl and mstst are
 * three non-memory registers that control hardware operation.
 * The actual locations in memory come from the system.k file, whereas the
 * three non-memory registers are understood directly by the assembler.
 * The details of the data loaded to these dm locations is given in the
 * Analog Devices "ADSP-2100 Family Users Manual."
 *****************************************************************************}

	ax0 = 0;			{ No wait states are used }
	dm(DM_Wait_Reg)=ax0;
        ax0 = b#0001000000000000;	{ Enable SPORT0 }
        dm(System_Control_Reg) = ax0;
        ifc = b#0000000011111111;	{ Clear any pending interrupt }
        nop;
	icntl = 0;			{ Interrupts level sensitive, no nest }
	mstat = b#1000000;		{ Primary data registers }
	rts;

{*****************************************************************************
 * Start up initialization of the AD1847 Codec.  This requires some effort to
 * simulate, and it is easiest to skip this part when running the sim21
 * simulator.  This method doesn't support all testing, but is OK for most
 * purposes.
 * This part of the program is the "background" routine during the
 * initialization of the AD1847 Codec.  The SPORT0 Tx interrupt is enabled.
 * The transmission of data to the Codec will program the
 * Codec control registers.  See the AD1847 data sheet for the myriads of
 * details involved.
 *****************************************************************************}
init1:	{ AD 1847 Codec Initialization
	  The stat_flag=0 will prevent reinit of AD1847 from SPORT0 TX ints }
	ax0 = 1;			{ Set flag so control words will be }
	dm(stat_flag) = ax0;		{   be loaded by next_cmd routine }
	imask = b#0001000000;		{ Enable SPORT0 Tx int, disable others}
	ax0 = dm(i6, m6);		{ Start interrupt }
	tx0 = ax0;
check_init:
        ax0 = dm (stat_flag);		{ Wait for entire initialization }
        af = pass ax0;			{   command buffer (init_cmds) to be }
        if ne jump check_init;		{   sent to the codec by next_cmd   }
        ay0 = 2;
check_acih:
        ax0 = dm (rx_buf);		{ Once initialized, wait for codec }
        ar = ax0 and ay0;		{   to come out of autocalibration }
        if eq jump check_acih;		{ Wait for bit set }
check_aci1:
        ax0 = dm (rx_buf);		{ Wait for bit clear }
        ar = ax0 and ay0;
        if ne jump check_aci1;
	idle;				{ Sit and wait for an interrupt }

        ay0 = 0xbf3f;			{ Unmute left DAC }
        ax0 = dm (init_cmds + 6);
        ar = ax0 AND ay0;
        dm (tx_buf) = ar;
        idle;

        ax0 = dm (init_cmds + 7);	{ Unmute right DAC }
        ar = ax0 AND ay0;
        dm (tx_buf) = ar;
        idle;

	ax0 = 0xc901;			{ Clear autocalibration request }
	dm(tx_buf)=ax0;
	idle;

        ax0 = 0x8000;			{ Clear over-range flags }
        dm (tx_buf) = ax0;
        idle;

        ifc = b#00000011111111;		{ Clear any pending interrupt }
        nop;
        imask = b#0000100000;		{ Enable RX0 interrupt, now our only }
	rts;


{************************  NEXT_CMD  ****************************************
 * Transmit Interrupt Service Routine used for Codec initialization
 * DAG 5 registers used here, but are available after this routine is run.
 * The commands are loaded here into the command location dm(tx_buf+0) where
 * they will be transferred to the Codec.
 * Adapted from EZ-KIT Lite routines.
 ****************************************************************************}
next_cmd:
        ena sec_reg;			{ Get other set of registers }
        ax0 = dm (i5, m5);		{ Get next control word and }
        dm (tx_buf+0) = ax0;		{   place in command word }
        ax0 = i5;			{ Now see if finished by comparing }
        ay0 = ^init_cmds;		{   the i5 address with the starting}
        ar = ax0 - ay0;			{   i5 value. Circ buff brings here}
        if gt rti;                  	{ Exit if more control words waiting }
        ax0 = 0x8000;			{ else clear the Mode Change Enable }
        dm (tx_buf+0) = ax0;		{   (MCE) bit in the Codec cntrl word}
        ax0 = 0;			{ Flag the completion by clearing }
        dm (stat_flag) = ax0;		{   stat_flag, stopping calls to here }
	dis sec_reg;
        rti;

.endmod;
